<?php

namespace Drupal\blazy;

/**
 * Provides common field formatter-related methods: Blazy, Slick.
 */
class BlazyFormatter extends BlazyManager {

  /**
   * Returns the field formatter settings inherited by child elements.
   *
   * @param array $build
   *   The array containing: settings, or potential optionset for extensions.
   * @param object $items
   *   The items to prepare settings for.
   * @param object $entity
   *   The entity this field belongs to.
   */
  public function buildSettings(array &$build, $items, $entity) {
    $settings       = &$build['settings'];
    $count          = count($items);
    $entity_type_id = $settings['entity_type_id'];
    $entity_id      = $settings['entity_id'];
    $bundle         = $settings['bundle'];
    $field_name     = $settings['field_name'];
    $field_clean    = str_replace('field_', '', $field_name);
    $view_mode      = empty($settings['current_view_mode']) ? '_custom' : $settings['current_view_mode'];
    $namespace      = $settings['namespace'] = empty($settings['namespace']) ? 'blazy' : $settings['namespace'];
    $id             = isset($settings['id']) ? $settings['id'] : '';
    $gallery_id     = "{$namespace}-{$entity_type_id}-{$bundle}-{$field_clean}-{$view_mode}";
    $id             = Blazy::getHtmlId("{$gallery_id}-{$entity_id}", $id);
    $switch         = empty($settings['media_switch']) ? '' : $settings['media_switch'];
    $internal_path  = entity_uri($entity_type_id, $entity);
    $langcode       = $settings['langcode'];

    // Pass settings to child elements.
    $settings['cache_metadata'] = ['keys' => [$id, $count, $langcode]];
    $settings['content_url']    = isset($internal_path['path']) ? $internal_path['path'] : '';
    $settings['count']          = $count;
    $settings['gallery_id']     = str_replace('_', '-', $gallery_id . '-' . $switch);
    $settings['id']             = $id;
    $settings['lightbox']       = ($switch && in_array($switch, $this->getLightboxes())) ? $switch : FALSE;
    $settings['entity']         = empty($settings['lightbox']) ? NULL : $entity;

    // Don't bother with Vanilla on.
    if (!empty($settings['vanilla'])) {
      $settings = array_filter($settings);
      return;
    }

    // Don't bother if using Responsive image.
    $settings['breakpoints'] = isset($settings['breakpoints']) && empty($settings['responsive_image_style']) ? $settings['breakpoints'] : [];
    if (!empty($settings['breakpoints'])) {
      $this->cleanUpBreakpoints($settings);
    }

    $settings['caption']    = empty($settings['caption']) ? [] : array_filter($settings['caption']);
    $settings['background'] = empty($settings['responsive_image_style']) && !empty($settings['background']);
    $settings['resimage']   = function_exists('picture_mapping_load') && $this->config('responsive_image', FALSE) && !empty($settings['responsive_image_style']);
    $settings['blazy']      = $settings['resimage'] || !empty($settings['blazy']);

    // At D7, BlazyFilter can only attach globally, prevents blocking.
    // Allows lightboxes to provide its own optionsets.
    if ($switch) {
      $settings[$switch] = empty($settings[$switch]) ? $switch : $settings[$switch];
    }

    // Let Blazy handle CSS background as Slick's background is deprecated.
    if ($settings['background']) {
      $settings['blazy'] = TRUE;
    }

    if ($settings['blazy']) {
      $settings['lazy'] = 'blazy';
    }

    // Aspect ratio isn't working with Responsive image, yet.
    // However allows custom work to get going with an enforced.
    $ratio = FALSE;
    if (!empty($settings['ratio'])) {
      $ratio = empty($settings['responsive_image_style']);
      if ($settings['ratio'] == 'enforced' || $settings['background']) {
        $ratio = TRUE;
      }
    }

    $settings['ratio'] = $ratio ? $settings['ratio'] : FALSE;

    // Pass first item to optimize sizes and build colorbox/zoom-like gallery.
    if (empty($settings['first_item']) && in_array($settings['field_type'], ['file', 'image']) && $items[0]) {
      $item = (object) $items[0];
      $settings['first_item'] = $item;
      $settings['first_uri'] = $item->uri;
    }

    // Sets dimensions once, if cropped, to reduce costs with ton of images.
    // This is less expensive than re-defining dimensions per image.
    if (!empty($settings['first_item']) && !empty($settings['image_style']) && !$settings['resimage']) {
      $this->setDimensionsOnce($settings);
    }

    // @todo remove once sub-modules updated to use first_ things.
    unset($settings['item'], $settings['uri']);

    // Sniffs for Views to allow block__no_wrapper, views_no_wrapper, etc.
    if (function_exists('views_get_current_view') && $view = views_get_current_view()) {
      $settings['view_name'] = $view->name;
      $settings['current_view_mode'] = $view->current_display;
    }

    drupal_alter('blazy_settings', $build, $items);

    // Done at top level works, prevents leaking to child for few settings.
    if ($namespace == 'blazy') {
      unset($settings['first_item']);
    }
  }

}
