<?php

namespace Drupal\blazy;

/**
 * Implements a public facing blazy manager.
 *
 * A few modules re-use this: GridStack, Mason, Slick...
 */
class BlazyManager extends BlazyManagerBase {

  /**
   * {@inheritdoc}
   */
  public function config($key = '', $default = NULL, $id = 'blazy.settings', array $defaults = []) {
    return parent::config($key, $default, $id, $id == 'blazy.settings' ? BlazyDefault::formSettings() : $defaults);
  }

  /**
   * {@inheritdoc}
   */
  public function typecast(array &$config, $id = 'blazy.settings') {
    if ($id == 'blazy.settings') {
      $defaults = BlazyDefault::formSettings();
      foreach ($defaults as $key => $value) {
        if (isset($config[$key])) {
          if ($key == 'blazy') {
            foreach ($defaults[$key] as $k => $v) {
              settype($config[$key][$k], gettype($v));
            }
          }
          elseif ($key == 'filters') {
            foreach ($defaults[$key] as $k => $v) {
              // Has nested array [filters][format][grid|column|media_switch].
              foreach ($config[$key] as $sk => $format) {
                foreach ($format as $ssk => $ignore) {
                  settype($config[$key][$sk][$ssk], gettype($v));
                }
              }
            }
          }
          else {
            settype($config[$key], gettype($value));
          }
        }
      }
    }
  }

  /**
   * Returns the enforced content, or image using theme_blazy().
   *
   * The image_style property is not directly used by theme_blazy() but for
   * a quick reference such as from a Views style plugin rather than deep drill.
   * Will be removed if we figure out a better way, or no longer useful.
   *
   * @param array $build
   *   The array containing:
   *     - item: The image / file entity item object converted from array.
   *     - settings: An array of settings to instruct Blazy what to do.
   *     - captions: Optional captions for both inline, or lightbox as per name.
   *
   * @return array
   *   The alterable and renderable array of enforced content, or theme_blazy().
   */
  public function getBlazy(array $build = []) {
    // @todo remove, and make it optional to remove barriers.
    // @todo if (empty($build['item'])) {
    // @todo return [];
    // @todo }
    $item = $build['item'] = isset($build['item']) ? $build['item'] : NULL;
    $settings = &$build['settings'];
    $settings['image_style'] = isset($settings['image_style']) ? $settings['image_style'] : '';
    $settings['uri'] = (empty($settings['uri']) && $item && isset($item->uri)) ? $item->uri : $settings['uri'];

    $image = [
      '#theme'       => 'blazy',
      '#image_style' => $settings['image_style'],
      '#item'        => isset($settings['entity_type_id']) && $settings['entity_type_id'] == 'user' ? $build['item'] : [],
      '#build'       => $build,
      '#pre_render'  => ['blazy_pre_render'],
    ];

    drupal_alter('blazy', $image, $settings);
    return empty($settings['uri']) ? [] : $image;
  }

  /**
   * Builds the Blazy as a structured array ready for ::renderer().
   *
   * @param array $element
   *   The pre-rendered element.
   *
   * @return array
   *   The renderable array of pre-rendered element.
   */
  public function preRender(array $element) {
    $build = $element['#build'];
    unset($element['#build']);

    // Prepare the main image.
    $this->prepareImage($element, $build);

    // Fetch the newly modified settings.
    $settings = $element['#settings'];

    // Provides optional media video if so configured.
    // Does it look familiar, `module_load_include()`, only native?
    // This allows a hybrid of media switcher and quasi-lightbox like Zooming.
    if ($settings['use_media'] && empty($settings['_noiframe'])) {
      BlazyMedia::build($element);
    }

    // Provides optional link to content or lightboxes if so configured.
    if ($switch = $settings['media_switch']) {
      if ($switch == 'content' && $settings['content_url']) {
        $element['#url'] = $settings['content_url'];
      }
      elseif ($settings['lightbox']) {
        BlazyLightbox::build($element);
      }
    }

    return $element;
  }

  /**
   * Prepares the Blazy image as a structured array ready for ::renderer().
   *
   * @param array $element
   *   The renderable array being modified.
   * @param array $build
   *   The array of information containing the required Image or File item
   *   object, settings, optional container attributes.
   */
  protected function prepareImage(array &$element, array $build) {
    $item = $build['item'];
    $settings = $build['settings'];
    $settings += BlazyDefault::itemSettings();
    $picture = !empty($settings['resimage']);

    // Decides if to use image or video, or both.
    $settings['_api'] = TRUE;
    $settings['use_media'] = $settings['embed_url'] && in_array($settings['type'], ['audio', 'video']);
    $settings['use_image'] = !empty($settings['uri']);
    $settings['ratio'] = $settings['ratio'] ? str_replace(':', '', $settings['ratio']) : FALSE;

    foreach (BlazyDefault::themeAttributes() as $key) {
      $key = $key . '_attributes';
      $build[$key] = isset($build[$key]) ? $build[$key] : [];
    }

    $attributes = isset($build['attributes']) ? $build['attributes'] : [];
    $item_attributes = $build['item_attributes'];
    $url_attributes = $build['url_attributes'];

    // Unlike D8, we have no free $item->_attributes from RDF, provide one here.
    // With or without rdf enabled, no need to check for module_exists().
    if ($settings['type'] == 'image') {
      $item_attributes['typeof'] = ['foaf:Image'];
    }

    // Extract field item attributes for the theme function, and unset them
    // from the $item so that the field template does not re-render them.
    if ($item && isset($item->_attributes)) {
      $item_attributes += $item->_attributes;
      unset($item->_attributes);
    }

    // Image style modifier can be multi-style images such as GridStack.
    $settings['image_url'] = empty($settings['image_url']) ? file_create_url($settings['uri']) : $settings['image_url'];
    if ($settings['image_style']) {
      $settings['image_url'] = image_style_url($settings['image_style'], $settings['uri']);

      // Only re-calculate dimensions if not cropped, nor already set.
      if (empty($settings['_dimensions'])) {
        $settings = array_merge($settings, Blazy::transformDimensions($settings['image_style'], $item));
      }
    }

    // Aspect ratio to fix layout reflow with lazyloaded images responsively.
    // This is outside 'lazy' to allow non-lazyloaded iframes use this too.
    if ($settings['width'] && !$picture) {
      if ($settings['ratio']) {
        $attributes['class'][] = 'media--ratio media--ratio--' . $settings['ratio'];

        if (in_array($settings['ratio'], ['enforced', 'fluid'])) {
          // If "lucky", Blazy/ Slick Views galleries may already set this once.
          // Lucky when you don't flatten out the Views output earlier.
          $padding = empty($settings['padding_bottom']) ? round((($settings['height'] / $settings['width']) * 100), 2) : $settings['padding_bottom'];
          $attributes['style'] = 'padding-bottom: ' . $padding . '%';
          $settings['_breakpoint_ratio'] = $settings['ratio'];

          // Views rewrite results or Twig inline_template may strip out `style`
          // attributes, provide hint to JS.
          $attributes['data-ratio'] = $padding;
        }
      }

      // Only output dimensions for non-responsive images.
      // Respects hand-coded image attributes.
      if (!isset($item_attributes['width'])) {
        $item_attributes['height'] = $settings['height'];
        $item_attributes['width'] = $settings['width'];
      }
    }

    // The fallback must run as fallback, also for Picture.
    if ($item) {
      foreach (['width', 'height', 'alt', 'title'] as $key) {
        if ($item && isset($item->{$key})) {
          // Respects hand-coded image attributes, image style, and set once.
          if (array_key_exists($key, $item_attributes)) {
            continue;
          }

          // Do not output an empty 'title' attribute.
          if ($key == 'title' && (strlen($item->title) != 0)) {
            $item_attributes['title'] = $item->title;
          }
          // Ensures to not override dimensions set once, or via image_style.
          elseif (!isset($item_attributes[$key])) {
            $item_attributes[$key] = $item->{$key};
          }
        }
      }
    }

    // Picture integration.
    if ($picture) {
      if ($mappings = picture_mapping_load($settings['responsive_image_style'])) {
        $settings['picture'] = picture_get_mapping_breakpoints($mappings, $settings['image_style']);
        $attributes['class'][] = 'media--picture';
      }
    }

    if (empty($settings['picture'])) {
      // Supports both lazyloaded or regular image.
      $item_attributes['class'][] = 'media__element';

      if ($settings['lazy']) {
        $item_attributes['src'] = Blazy::PLACEHOLDER;
        $attributes['class'][] = 'media--loading';

        // Attach data attributes to either IMG tag, or DIV container.
        // Supports Blazy CSS background lazyloading on DIV element.
        if ($settings['background']) {
          Blazy::buildBreakpointAttributes($attributes, $settings, $item);
          $attributes['class'][] = 'media--background';
        }
        else {
          Blazy::buildBreakpointAttributes($item_attributes, $settings, $item);
        }

        // Multi-breakpoint aspect ratio only applies if lazyloaded.
        if (!empty($settings['blazy_data']['dimensions'])) {
          $attributes['data-dimensions'] = drupal_json_encode($settings['blazy_data']['dimensions']);
        }
      }
    }

    $attributes['class'][] = 'media--' . $settings['type'];
    if (!empty($settings['media_switch'])) {
      $attributes['class'][] = 'media--switch media--switch--' . str_replace('_', '-', $settings['media_switch']);
    }

    // Thumbnails.
    // With CSS background, IMG may be empty, add thumbnail to the container.
    // Supports unique thumbnail different from main image, such as logo for
    // thumbnail and main image for company profile.
    if (!empty($settings['thumbnail_uri'])) {
      $attributes['data-thumb'] = file_create_url($settings['thumbnail_uri']);
    }
    elseif (!empty($settings['thumbnail_style'])) {
      $attributes['data-thumb'] = image_style_url($settings['thumbnail_style'], $settings['uri']);
    }

    // Provides extra attributes as needed, excluding url, item, done above.
    foreach (['caption', 'media', 'wrapper'] as $key) {
      $element["#$key" . '_attributes'] = $build[$key . '_attributes'];
    }

    $captions = empty($build['captions']) ? [] : $this->buildCaption($build['captions'], $settings);
    if ($captions) {
      $element['#caption_attributes']['class'][] = $settings['item_id'] . '__caption';
    }

    // Provides data for the renderable elements.
    $element['#item']            = $item;
    $element['#captions']        = $captions;
    $element['#attributes']      = $attributes;
    $element['#item_attributes'] = $item_attributes;
    $element['#url_attributes']  = $url_attributes;
    $element['#settings']        = $settings;
  }

  /**
   * Build captions for both old image, or media entity.
   */
  public function buildCaption(array $captions, array $settings) {
    $content = [];
    foreach ($captions as $key => $caption_content) {
      if ($caption_content) {
        $content[$key]['content'] = $caption_content;
        $content[$key]['tag'] = strpos($key, 'title') !== FALSE ? 'h2' : 'div';
        $class = $key == 'alt' ? 'description' : str_replace('field_', '', $key);
        $content[$key]['attributes'] = [];
        $content[$key]['attributes']['class'][] = $settings['item_id'] . '__caption--' . str_replace('_', '-', $class);
      }
    }

    return $content ? ['inline' => $content] : [];
  }

  /**
   * Returns the contents using theme_field(), or theme_item_list().
   *
   * @param array $build
   *   The array containing: settings, children elements, or optional items.
   *
   * @return array
   *   The alterable and renderable array of contents.
   */
  public function build(array $build = []) {
    $settings = $build['settings'];
    $settings['_grid'] = isset($settings['_grid']) ? $settings['_grid'] : (!empty($settings['style']) && !empty($settings['grid']));

    // If not a grid, pass the items as regular index children to theme_field().
    // @todo #pre_render doesn't work if called from Views results.
    if (empty($settings['_grid'])) {
      $settings = $this->prepareBuild($build);
      $build['#blazy'] = $settings;
      $build['#attached'] = $this->attach($settings);
    }
    else {
      $build = [
        '#build'      => $build,
        '#settings'   => $settings,
        '#pre_render' => ['blazy_pre_render_build'],
      ];
    }

    drupal_alter('blazy_build', $build, $settings);
    return $build;
  }

  /**
   * Builds the Blazy outputs as a structured array ready for ::renderer().
   */
  public function preRenderBuild(array $element) {
    $build = $element['#build'];
    unset($element['#build']);

    $settings = $this->prepareBuild($build);

    $element = BlazyGrid::build($build, $settings);
    $element['#attached'] = $this->attach($settings);

    return $element;
  }

  /**
   * Prepares Blazy outputs, extract items, and returns updated $settings.
   */
  public function prepareBuild(array &$build) {
    // If children are stored within items, reset.
    $settings = isset($build['settings']) ? $build['settings'] : [];
    $build = isset($build['items']) ? $build['items'] : $build;

    // Supports Blazy multi-breakpoint images if provided, updates $settings.
    // Cases: Blazy within Views gallery, or references without direct image.
    if (!empty($settings['first_image']) && !empty($settings['check_blazy'])) {
      // Views may flatten out the array, bail out.
      // What we do here is extract the formatter settings from the first found
      // image and pass its settings to this container so that Blazy Grid which
      // lacks of settings may know if it should load/ display a lightbox, etc.
      // Lightbox should work without `Use field template` checked.
      if (is_array($settings['first_image'])) {
        $this->isBlazy($settings, $settings['first_image']);
      }
    }

    unset($build['items'], $build['settings']);
    return $settings;
  }

}
